package org.apache.xindice.server.rpc.messages;

/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * $Id: Query.java,v 1.3 2002/11/22 10:15:35 vladimir Exp $
 */

import org.apache.xindice.core.Collection;
import org.apache.xindice.core.data.NodeSet;
import org.apache.xindice.server.rpc.RPCDefaultMessage;
import org.apache.xindice.xml.NamespaceMap;
import org.apache.xindice.xml.TextWriter;
import org.apache.xindice.xml.dom.DBNode;
import org.apache.xindice.xml.dom.DocumentImpl;

import org.w3c.dom.*;

import java.util.Enumeration;
import java.util.Hashtable;

   /**
    * Executes a query against a document or collection
    */

public class Query extends RPCDefaultMessage {   
   
   public Hashtable execute(Hashtable message) throws Exception {

      if(!message.containsKey(COLLECTION)) {
         throw new Exception(MISSING_COLLECTION_PARAM);
      }

      if(!message.containsKey(TYPE)) {
         throw new Exception(MISSING_TYPE_PARAM);
      }

      if(!message.containsKey(QUERY)) {
         throw new Exception(MISSING_QUERY_PARAM);
      }

      Collection col = getCollection( (String) message.get(COLLECTION) );
      NodeSet ns = null;

      if (message.containsKey( NAME )) {

         ns = col.queryDocument( (String)message.get(TYPE), (String)message.get(QUERY), mapNamespaces( (Hashtable)message.get(NAMESPACES) ), (String)message.get(NAME) );

      } else {

         ns = col.queryCollection( (String)message.get(TYPE), (String)message.get(QUERY), mapNamespaces( (Hashtable)message.get(NAMESPACES) ) );
      }

      Hashtable result = new Hashtable();
      result.put(RESULT, queryWrapper( ns ));
      return result;
   }

   /**
    * Maps a Hashtable containing namespace definitions into a Xindice
    * NamespaceMap.
    *
    * @param namespaces
    * @return A NamespaceMap
    */
   protected NamespaceMap mapNamespaces( Hashtable namespaces ) {
      NamespaceMap nsMap = null;
      if ( namespaces.size() > 0 ) {
         nsMap = new NamespaceMap();

         Enumeration keys = namespaces.keys();
         while ( keys.hasMoreElements() ) {
            String key = ( String ) keys.nextElement();
            nsMap.setNamespace( key, ( String ) namespaces.get( key ) );
         }
      }

      return nsMap;
   }

   /**
    * Adds additional meta data to the query response and turns it into a
    * Document.
    *
    * @param ns The NodeSet containing the query results.
    * @return the result set as an XML document
    * @exception Exception
    */
   private String queryWrapper( NodeSet ns ) throws Exception {
      // Turn the NodeSet into a document.
      DocumentImpl doc = new DocumentImpl();

      Element root = doc.createElement( "result" );
      doc.appendChild( root );
      int count = 0;
      while ( ns != null && ns.hasMoreNodes() ) {
         Node n = ns.getNextNode();

         if ( n.getNodeType() == Node.DOCUMENT_NODE ) {
            n = ( ( Document ) n ).getDocumentElement();
         }

         if ( n instanceof DBNode ) {
            ( ( DBNode ) n ).expandSource();
         }

         root.appendChild( doc.importNode( n, true ) );
         count++;
      }

      root.setAttribute( "count", Integer.toString( count ) );

      return TextWriter.toString( doc );
   }

}
