/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * $Id: CollectionTest.java,v 1.3 2002/10/28 08:39:20 vladimir Exp $
 */

package org.apache.xindice.integration.client.basic;

import org.apache.xindice.integration.client.AbstractXmlDbClientTest;
import org.apache.xindice.integration.client.XmlDbClientSetup;

import org.xmldb.api.base.XMLDBException;
import org.xmldb.api.base.Collection;

import java.util.Arrays;
import java.util.List;
import java.util.Vector;

/**
 * @version $Revision: 1.3 $, $Date: 2002/10/28 08:39:20 $
 * @author Vladimir R. Bossicard <vladimir@apache.org>
 */
public class CollectionTest
      extends AbstractXmlDbClientTest {

   /* This is just to make sure that the system is up and running. */
   public void testListInitialCollections()
         throws Exception {
      String[] collections = this.client.listCollections(XmlDbClientSetup.INSTANCE_NAME);
      assertEquals(2, collections.length);
      List col = new Vector(Arrays.asList(collections));
      assertTrue(col.contains(XmlDbClientSetup.TEST_COLLECTION_NAME));
      assertTrue(col.contains("system"));
   }

   public void testCreateNestedCollection()
         throws Exception {
      this.client.createCollection(TEST_COLLECTION_PATH, "nested");
      assertEquals(1, this.client.countCollections(TEST_COLLECTION_PATH));

      this.client.createCollection(TEST_COLLECTION_PATH + "/nested", "child2");
      this.client.createCollection(TEST_COLLECTION_PATH + "/nested", "child3");
      assertEquals(2, this.client.countCollections(TEST_COLLECTION_PATH + "/nested"));

      this.client.dropCollection(TEST_COLLECTION_PATH + "/nested", "child2");
      assertEquals(1, this.client.countCollections(TEST_COLLECTION_PATH + "/nested"));

      this.client.dropCollection(TEST_COLLECTION_PATH + "/nested", "child3");
      assertEquals(0, this.client.countCollections(TEST_COLLECTION_PATH + "/nested"));

      this.client.dropCollection(TEST_COLLECTION_PATH, "nested");
   }

   public void testCreateDuplicateCollection()
         throws Exception {
      this.client.createCollection(TEST_COLLECTION_PATH, "duplicate");
      try {
         this.client.createCollection(TEST_COLLECTION_PATH, "duplicate");
      } catch (XMLDBException e) {
         //assertTrue( e.getMessage().lastIndexOf( "Duplicate Collection" ) > 0 );
      }

      this.client.dropCollection(TEST_COLLECTION_PATH, "duplicate");
   }

   public void testCreateCollectionEmptyName()
         throws Exception {
      try {
         this.client.createCollection(TEST_COLLECTION_PATH, "");
      } catch (XMLDBException e) {
         //assertTrue( e.getMessage().lastIndexOf( "Duplicate Collection" ) > 0 );
         return;
      }
   }

   public void testCreateCollectionNullName()
         throws Exception {
      try {
         this.client.createCollection(TEST_COLLECTION_PATH, null);
      } catch (XMLDBException e) {
         //assertTrue( e.getMessage().lastIndexOf( "Duplicate Collection" ) > 0 );
         return;
      }
   }

   public void testDropCollectionTwice()
         throws Exception {
      this.client.createCollection(TEST_COLLECTION_PATH, "droptwice");

      this.client.dropCollection(TEST_COLLECTION_PATH, "droptwice");
      try {
         this.client.dropCollection(TEST_COLLECTION_PATH, "droptwice");
      } catch (XMLDBException e) {
         //assertTrue( e.getMessage().lastIndexOf( "could not be found" ) > 0 );
         return;
      }
   }

   public void testDropCollectionNullName()
         throws Exception {
      try {
         this.client.dropCollection(TEST_COLLECTION_PATH, null);
      } catch (XMLDBException e) {
         //assertTrue( e.getMessage().lastIndexOf( "could not be found" ) > 0 );
         return;
      }
   }

   public void testDropCollectionEmptyName()
         throws Exception {
      try {
         this.client.dropCollection(TEST_COLLECTION_PATH, "");
      } catch (XMLDBException e) {
         //assertTrue( e.getMessage().lastIndexOf( "could not be found" ) > 0 );
         return;
      }
   }

   public void testGetCollectionCount()
         throws Exception {
      this.client.createCollection(TEST_COLLECTION_PATH, "count");
      assertEquals(1, this.client.countCollections(TEST_COLLECTION_PATH));

      this.client.dropCollection(TEST_COLLECTION_PATH, "count");
      assertEquals(0, this.client.countCollections(TEST_COLLECTION_PATH));
   }

   public void testListCollections()
         throws Exception {
      String[] collections = this.client.listCollections(TEST_COLLECTION_PATH);
      assertEquals(0, collections.length);

      this.client.createCollection(TEST_COLLECTION_PATH, "child1");
      this.client.createCollection(TEST_COLLECTION_PATH, "child2");

      collections = this.client.listCollections(TEST_COLLECTION_PATH);
      assertEquals(2, collections.length);
      List col = new Vector(Arrays.asList(collections));
      assertTrue(col.contains("child1"));
      assertTrue(col.contains("child2"));

      this.client.dropCollection(TEST_COLLECTION_PATH, "child1");
      collections = this.client.listCollections(TEST_COLLECTION_PATH);
      assertEquals(1, collections.length);
      assertEquals("child2", collections[0]);

      this.client.dropCollection(TEST_COLLECTION_PATH, "child2");

      collections = this.client.listCollections(TEST_COLLECTION_PATH);
      assertEquals(0, collections.length);
   }

   public void testGetName()
         throws Exception {
      this.client.createCollection(TEST_COLLECTION_PATH, "getname");
      assertEquals("getname", this.client.getName(TEST_COLLECTION_PATH + "/getname"));
      this.client.dropCollection(TEST_COLLECTION_PATH, "getname");
   }

   public void testGetParentCollection()
         throws Exception {
      Collection col = this.client.createCollection(TEST_COLLECTION_PATH, "childcol");

      Collection parent = col.getParentCollection();
      assertNotNull(parent);
      assertEquals("current", parent.getName());

      parent = parent.getParentCollection();
      assertNotNull(parent);
      assertEquals("testing", parent.getName());

      parent = parent.getParentCollection();
      assertNull(parent);

      this.client.dropCollection(TEST_COLLECTION_PATH, "childcol");
   }

   public void testGetCollectionUnknown()
         throws Exception {
      assertNull(this.client.getCollection(TEST_COLLECTION_PATH + "/unknown"));
   }

   public void testChildCollectionCount()
         throws Exception {
      Collection col = this.client.createCollection(TEST_COLLECTION_PATH, "childcol");
      assertEquals(0, col.getChildCollectionCount());

      this.client.createCollection(TEST_COLLECTION_PATH, "colcount");
      assertEquals(0, col.getChildCollectionCount());

      this.client.dropCollection(TEST_COLLECTION_PATH, "colcount");
      assertEquals(0, col.getChildCollectionCount());
   }

}
