/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * $Id: DocumentTest.java,v 1.3 2002/10/28 08:39:20 vladimir Exp $
 */

package org.apache.xindice.integration.client.basic;

import org.apache.xindice.integration.client.AbstractXmlDbClientTest;

import org.xmldb.api.base.XMLDBException;

/**
 * @version $Revision: 1.3 $, $Date: 2002/10/28 08:39:20 $
 * @author Vladimir R. Bossicard <vladimir@apache.org>
 */
public class DocumentTest
      extends AbstractXmlDbClientTest {

   public void testInsertDocument()
         throws Exception {
      this.client.insertDocument(TEST_COLLECTION_PATH, "testdoc", "<test>test data</test>");
      assertEquals(1, this.client.countDocument(TEST_COLLECTION_PATH));

      this.client.removeDocument(TEST_COLLECTION_PATH, "testdoc");
      assertEquals(0, this.client.countDocument(TEST_COLLECTION_PATH));
   }

   public void testRemoveDocumentInvalidName()
         throws Exception {
      try {
         this.client.removeDocument(TEST_COLLECTION_PATH, "invalidname");
      } catch (XMLDBException e) {
         //assertTrue( e.getMessage().lastIndexOf( "Duplicate Collection" ) > 0 );
         return;
      }
   }

   public void testRemoveDocumentEmptyName()
         throws Exception {
      try {
         this.client.removeDocument(TEST_COLLECTION_PATH, "");
      } catch (XMLDBException e) {
         //assertTrue( e.getMessage().lastIndexOf( "Duplicate Collection" ) > 0 );
         return;
      }
   }

   public void testRemoveDocumentNullName()
         throws Exception {
      try {
         this.client.removeDocument(TEST_COLLECTION_PATH, null);
      } catch (XMLDBException e) {
         //assertTrue( e.getMessage().lastIndexOf( "Duplicate Collection" ) > 0 );
         return;
      }
   }

   public void testGetDocument()
         throws Exception {
      String testDocument = "<?xml version=\"1.0\"?>\n<data><test>test data</test></data>";
      this.client.insertDocument(TEST_COLLECTION_PATH, "doc1", testDocument);

      String doc = this.client.getDocument(TEST_COLLECTION_PATH, "doc1");
      assertNotNull(doc);
      assertEquals(testDocument, doc);

      this.client.removeDocument(TEST_COLLECTION_PATH, "doc1");
   }

   public void testGetInexistantDocument()
         throws Exception {
      String doc = this.client.getDocument(TEST_COLLECTION_PATH, "ghostdoc");
      assertNull(doc);
   }

   public void testGetDocumentEmptyName()
         throws Exception {
      String doc = this.client.getDocument(TEST_COLLECTION_PATH, "");
      assertNull(doc);
   }

   public void testGetDocumentNullName()
         throws Exception {
      String doc = this.client.getDocument(TEST_COLLECTION_PATH, null);
      assertNull(doc);
   }

   public void testListDocuments()
         throws Exception {
      String[] documents = this.client.listDocuments(TEST_COLLECTION_PATH);
      assertEquals(0, this.client.listDocuments(TEST_COLLECTION_PATH).length);

      this.client.insertDocument(TEST_COLLECTION_PATH, "doc1", "<data>\n<test>\ntest data</test>\n</data>");
      documents = this.client.listDocuments(TEST_COLLECTION_PATH);
      assertEquals(1, documents.length);
      assertEquals("doc1", documents[0]);

      this.client.insertDocument(TEST_COLLECTION_PATH, "doc2", "<data>\n<test>\ntest data2</test>\n</data>");
      documents = this.client.listDocuments(TEST_COLLECTION_PATH);
      assertEquals(2, documents.length);
      assertEquals("doc1", documents[0]);
      assertEquals("doc2", documents[1]);

      this.client.removeDocument(TEST_COLLECTION_PATH, "doc1");
      documents = this.client.listDocuments(TEST_COLLECTION_PATH);
      assertEquals(1, documents.length);
      assertEquals("doc2", documents[0]);

      this.client.removeDocument(TEST_COLLECTION_PATH, "doc2");
      documents = this.client.listDocuments(TEST_COLLECTION_PATH);
      assertEquals(0, documents.length);
   }

   public void testGetDocumentCount()
         throws Exception {
      assertEquals(0, this.client.countDocument(TEST_COLLECTION_PATH));

      this.client.insertDocument(TEST_COLLECTION_PATH, "doc1", "<data>\n<test>\ntest data</test>\n</data>");
      assertEquals(1, this.client.countDocument(TEST_COLLECTION_PATH));

      this.client.insertDocument(TEST_COLLECTION_PATH, "doc2", "<data>\n<test>\ntest data2</test>\n</data>");
      assertEquals(2, this.client.countDocument(TEST_COLLECTION_PATH));

      this.client.removeDocument(TEST_COLLECTION_PATH, "doc1");
      assertEquals(1, this.client.countDocument(TEST_COLLECTION_PATH));

      this.client.removeDocument(TEST_COLLECTION_PATH, "doc2");
      assertEquals(0, this.client.countDocument(TEST_COLLECTION_PATH));
   }

}
