/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * $Id: XUpdateQueryTest.java,v 1.1 2002/11/28 08:12:26 vladimir Exp $
 */

package org.apache.xindice.integration.client.services;

import org.apache.xindice.integration.client.AbstractXmlDbClientTest;

import org.xmldb.api.modules.XUpdateQueryService;
import org.xmldb.api.base.Collection;

/**
 * @version $Revision: 1.1 $, $Date: 2002/11/28 08:12:26 $
 * @author Vladimir R. Bossicard <vladimir@apache.org>
 */
public class XUpdateQueryTest
        extends AbstractXmlDbClientTest {

    public void testUpdateDocument()
            throws Exception {
        String document = "<?xml version=\"1.0\"?>" +
                "<person>" +
                "<fname>John</fname>" +
                "<lname>Smith</lname>" +
                "<phone type=\"work\">563-456-7890</phone>" +
                "<phone type=\"home\">534-567-8901</phone>" +
                "<email type=\"home\">jsmith@somemail.com</email>" +
                "<email type=\"work\">john@lovesushi.com</email>" +
                "<address type=\"home\">34 S. Colon St.</address>" +
                "<address type=\"work\">9967 W. Shrimp Ave.</address>" +
                "</person>";

        this.client.insertDocument(TEST_COLLECTION_PATH, "xupdatedoc", document);
        assertEquals(1, this.client.countDocument(TEST_COLLECTION_PATH));

        String xupdate = "<xu:modifications version=\"1.0\"" +
                "      xmlns:xu=\"http://www.xmldb.org/xupdate\">" +
                "   <xu:remove select=\"/person/phone[@type = 'home']\"/>" +
                "   <xu:update select=\"/person/phone[@type = 'work']\">" +
                "       480-300-3003" +
                "   </xu:update>" +
                "</xu:modifications>";

        String updatedDocument = "<?xml version=\"1.0\"?>" +
                "<person>" +
                "<fname>John</fname>" +
                "<lname>Smith</lname>" +
                "<phone type=\"work\">408-300-3003</phone>" +
                "<email type=\"home\">jsmith@somemail.com</email>" +
                "<email type=\"work\">john@lovesushi.com</email>" +
                "<address type=\"home\">34 S. Colon St.</address>" +
                "<address type=\"work\">9967 W. Shrimp Ave.</address>" +
                "</person>";

        Collection col = this.client.getCollection(TEST_COLLECTION_PATH);
        XUpdateQueryService service = (XUpdateQueryService) col.getService("XUpdateQueryService", "1.0");

        long count = service.update(xupdate);
        assertEquals(1, count);

        String doc = this.client.getDocument(TEST_COLLECTION_PATH, "xupdatedoc");
        assertNotNull(doc);
        assertEquals(updatedDocument, doc);

        this.client.removeDocument(TEST_COLLECTION_PATH, "xupdatedoc");
        assertEquals(0, this.client.countDocument(TEST_COLLECTION_PATH));
    }

}
